<?php

/**
 * Manage Teams
 */

namespace Dog\Helper;

use ManiaControl\Players\Player;
use Dog\Exception\RoyalException;
use Exception;

class Teams
{
    private static array $availableTeams = [
        "Flamingo", "Pig", "Clown_Fish", "Fox", "Octopus", "Butterfly", "Crocodile", "Grasshopper", "Ladybug", "Macaw_Parrot",
        "Giraffe", "Bee", "Dolphin", "Peafowl", "Kangaroo", "Monkey", "Panda", "Zebra", "Rabbit", "Polar_Bear"
    ];

    private static array $pendingTeams = [];

    public static function Init(bool $is30TeamsEnabled): void
    {
        if ($is30TeamsEnabled) {
            array_push(self::$availableTeams, "Team21", "Team22", "Team23", "Team24", "Team25", "Team26", "Team27", "Team28", "Team29", "Team30");
        }

        foreach (self::$availableTeams as $team) {
            self::$pendingTeams[$team] = [];
        }
    }

    /** @throws \Dog\Exception\RoyalException */
    public static function AddPlayer(Player $player, string $team): void
    {
        if (self::GetTeamIdV2($team) < 0) {
            throw new Exception("Team $team does not exist.");
        }

        if (self::GetTeamIdV2($player) > -1) {
            throw new RoyalException("Player $player->nickname is already part of a team.");
        }
        self::$pendingTeams[$team][$player->index] = $player;
    }

    public static function RemovePlayer(Player $player): void
    {
        foreach (self::$availableTeams as $team) {
            if (array_key_exists($player->index, self::$pendingTeams[$team])) {
                unset(self::$pendingTeams[$team][$player->index]);
            }
        }
    }

    public static function Pending(): array
    {
        return self::$pendingTeams;
    }

    public static function GetTeamIdFromTeamName(string $team): int
    {
        $team = ucwords($team, "_");

        if (in_array($team, self::$availableTeams)) {
            return array_search($team, self::$availableTeams) + 1;
        }

        return -1;
    }

    /**
     * Return the team id for a given player or team name
     *
     * @param Player|string $param A Player obj or valid team name.
     * @return integer The team id or -1 if no team was found.
     */
    public static function GetTeamIdV2($param): int
    {
        if ($param instanceof Player) {
            foreach (self::$availableTeams as $team) {
                if (array_key_exists($param->index, self::$pendingTeams[$team])) {
                    return array_search($team, self::$availableTeams) + 1;
                }
            }
        } else {
            $team = ucwords($param, "_");

            if (in_array($team, self::$availableTeams)) {
                return array_search($team, self::$availableTeams) + 1;
            }
        }

        return -1;
    }

    /* public static function GetTeamId(Player $player): int
    {
        foreach (self::$availableTeams as $team) {
            if (array_key_exists($player->index, self::$pendingTeams[$team])) {
                return array_search($team, self::$availableTeams) + 1;
            }
        }
        return -1;
    } */

    public static function GetMembers(string $team): string
    {
        return implode(" | ", array_map(function ($player) {
            return $player->nickname;
        }, self::$pendingTeams[$team]));
    }

    /**
     * JSON string for RoyalTeams #Struct
     *
     * @return string
     */
    public static function toJSON(): string
    {
        $array = [];
        foreach (self::$availableTeams as $team) {
            array_push($array, '"' . $team . '":{"Members":"' . self::GetMembers($team) . '"}');
        }
        return (string) "{" . implode(",", $array) . "}";
    }

    public static function GetAvailableTeams(): array
    {
        return self::$availableTeams;
    }

    public static function Randomize(int $nbPlayersPerTeam, $players)
    {
        //TODO:
    }

    public static function GetTeamName(int $teamId)
    {
        return self::$availableTeams[$teamId - 1];
    }
}
